"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClusteredBrowser = void 0;
const poolpeteer_1 = require("poolpeteer");
const puppeteer_cluster_1 = require("puppeteer-cluster");
const browser_1 = require("./browser");
var RenderType;
(function (RenderType) {
    RenderType["CSV"] = "csv";
    RenderType["PNG"] = "png";
})(RenderType || (RenderType = {}));
const contextPerRenderKey = 'contextPerRenderKey';
class ClusteredBrowser extends browser_1.Browser {
    constructor(config, log, metrics) {
        super(config, log, metrics);
        this.getGroupId = (options) => {
            if (this.clusteringConfig.mode === contextPerRenderKey) {
                return `${options.domain}${options.renderKey}`;
            }
            return undefined;
        };
        this.clusteringConfig = config.clustering;
        this.concurrency = puppeteer_cluster_1.Cluster.CONCURRENCY_BROWSER;
        if (this.clusteringConfig.mode === 'context') {
            this.concurrency = puppeteer_cluster_1.Cluster.CONCURRENCY_CONTEXT;
        }
        if (this.clusteringConfig.mode === contextPerRenderKey) {
            this.concurrency = poolpeteer_1.Cluster.CONCURRENCY_CONTEXT_PER_REQUEST_GROUP;
        }
    }
    shouldUsePoolpeteer() {
        return this.clusteringConfig.mode === contextPerRenderKey;
    }
    async createCluster() {
        const launcherOptions = this.getLauncherOptions({});
        const clusterOptions = {
            concurrency: this.concurrency,
            workerShutdownTimeout: 5000,
            monitor: this.clusteringConfig.monitor,
            maxConcurrency: this.clusteringConfig.maxConcurrency,
            timeout: this.clusteringConfig.timeout * 1000,
            puppeteerOptions: launcherOptions,
        };
        // TODO use poolpeteer by default after initial release and testing (8.5?)
        if (this.shouldUsePoolpeteer()) {
            this.log.debug('Launching Browser cluster with poolpeteer');
            return poolpeteer_1.Cluster.launch(clusterOptions);
        }
        this.log.debug('Launching Browser cluster with puppeteer-cluster');
        return puppeteer_cluster_1.Cluster.launch(clusterOptions);
    }
    async start() {
        this.cluster = await this.createCluster();
        await this.cluster.task(async ({ page, data }) => {
            if (data.options.timezone) {
                // set timezone
                await page.emulateTimezone(data.options.timezone);
            }
            try {
                this.addPageListeners(page);
                switch (data.renderType) {
                    case RenderType.CSV:
                        return await this.exportCSV(page, data.options);
                    case RenderType.PNG:
                    default:
                        return await this.takeScreenshot(page, data.options);
                }
            }
            finally {
                this.removePageListeners(page);
            }
        });
    }
    async render(options) {
        this.validateImageOptions(options);
        return this.cluster.execute({ groupId: this.getGroupId(options), options, renderType: RenderType.PNG });
    }
    async renderCSV(options) {
        this.validateRenderOptions(options);
        return this.cluster.execute({ groupId: this.getGroupId(options), options, renderType: RenderType.CSV });
    }
}
exports.ClusteredBrowser = ClusteredBrowser;
//# sourceMappingURL=clustered.js.map