"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReusableBrowser = void 0;
const puppeteer = require("puppeteer");
const browser_1 = require("./browser");
class ReusableBrowser extends browser_1.Browser {
    constructor(config, log, metrics) {
        super(config, log, metrics);
    }
    async start() {
        const launcherOptions = this.getLauncherOptions({});
        this.browser = await puppeteer.launch(launcherOptions);
    }
    async render(options) {
        let context;
        let page;
        try {
            page = await this.withTimingMetrics(async () => {
                this.validateImageOptions(options);
                context = await this.browser.createIncognitoBrowserContext();
                return context.newPage();
            }, 'newPage');
            if (options.timezone) {
                // set timezone
                await page.emulateTimezone(options.timezone);
            }
            this.addPageListeners(page);
            return await this.takeScreenshot(page, options);
        }
        finally {
            if (page) {
                this.removePageListeners(page);
                await page.close();
            }
            if (context) {
                await context.close();
            }
        }
    }
    async renderCSV(options) {
        let context;
        let page;
        try {
            this.validateRenderOptions(options);
            context = await this.browser.createIncognitoBrowserContext();
            page = await context.newPage();
            if (options.timezone) {
                // set timezone
                await page.emulateTimezone(options.timezone);
            }
            this.addPageListeners(page);
            return await this.exportCSV(page, options);
        }
        finally {
            if (page) {
                this.removePageListeners(page);
                await page.close();
            }
            if (context) {
                await context.close();
            }
        }
    }
}
exports.ReusableBrowser = ReusableBrowser;
//# sourceMappingURL=reusable.js.map